<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="text"/>

  <xsl:template match="qhacc">
    begin transaction;
    <xsl:apply-templates/>
    commit;
  </xsl:template>

  <xsl:template match="preferences|namedtrans|jobs"/>

  <xsl:template match="journal">
    insert into journal(id, name, description) values (
    <xsl:value-of select="ID"/>,
    <xsl:text>'</xsl:text>
    <xsl:value-of select="NAME"/>
    <xsl:text>'</xsl:text>,
    <xsl:choose>
      <xsl:when test="DESCRIPTION!=''">
        <xsl:text>'</xsl:text>
        <xsl:value-of select="DESCRIPTION"/>
        <xsl:text>'</xsl:text>
      </xsl:when>
      <xsl:otherwise>null</xsl:otherwise>
    </xsl:choose> );
  </xsl:template>

  <xsl:template match="account[ID='0']"/>
  <xsl:template match="account[ID!='0']">
    insert into account( id,parentid, name, openingbalance, banknumber, notes, accounttypeid) values (
    <xsl:value-of select="ID"/>,
    <xsl:choose>
      <xsl:when test="PID!='0'">
        <xsl:text></xsl:text>
        <xsl:value-of select="PID"/>
        <xsl:text></xsl:text>
      </xsl:when>
      <xsl:otherwise>null</xsl:otherwise>
    </xsl:choose>,
    <xsl:text>'</xsl:text><xsl:value-of select="NAME"/><xsl:text>'</xsl:text>,
    <xsl:choose>
      <xsl:when test="SUM!=''">
        <xsl:value-of select="SUM"/>
      </xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>,
    <xsl:choose>
      <xsl:when test="NUM!=''">
        <xsl:text>'</xsl:text>
        <xsl:value-of select="NUM"/>
        <xsl:text>'</xsl:text>
      </xsl:when>
      <xsl:otherwise>null</xsl:otherwise>
    </xsl:choose>,
    <xsl:text>'</xsl:text><xsl:value-of select="INSTNAME"/><xsl:text>
</xsl:text><xsl:value-of select="INSTADDR1"/><xsl:text>
</xsl:text><xsl:value-of select="INSTADDR2"/><xsl:text>
</xsl:text><xsl:value-of select="INSTCITY"/><xsl:text>
</xsl:text><xsl:value-of select="INSTSTATE"/><xsl:text>
</xsl:text><xsl:value-of select="INSTZIP"/><xsl:text>
</xsl:text><xsl:value-of select="INSTPHONE"/><xsl:text>
</xsl:text><xsl:value-of select="INSTFAX"/><xsl:text>
</xsl:text><xsl:value-of select="INSTEMAIL"/><xsl:text>
</xsl:text><xsl:value-of select="INSTCONTACT"/><xsl:text>
</xsl:text><xsl:value-of select="INSTNOTES"/><xsl:text>
</xsl:text><xsl:text>'</xsl:text>,
    <xsl:value-of select="TYPE"/>+1);
  </xsl:template>

  <xsl:template match="transaction[TID=0]"/>
  <xsl:template match="transaction[TID!=0]">
    insert into transentry( id, typeid, num, date, payee, journalid ) values (
    <xsl:value-of select="TID"/>,
    <xsl:choose>
      <xsl:when test="TVOID='Y'">2</xsl:when>
      <xsl:otherwise>1</xsl:otherwise>
    </xsl:choose>,
    <xsl:choose>
      <xsl:when test="NUM!=''">
        <xsl:text>'</xsl:text>
        <xsl:value-of select="NUM"/>
        <xsl:text>'</xsl:text>
      </xsl:when>
      <xsl:otherwise>null</xsl:otherwise>
    </xsl:choose>,
    <xsl:text>'</xsl:text><xsl:value-of select="DATE"/><xsl:text>'</xsl:text>,
    <xsl:choose>
      <xsl:when test="PAYEE!=''">
        <xsl:text>'</xsl:text>
        <xsl:value-of select="PAYEE"/>
        <xsl:text>'</xsl:text>
      </xsl:when>
      <xsl:otherwise>null</xsl:otherwise>
    </xsl:choose>,
    <xsl:value-of select="JOURNALID"/>);
  </xsl:template>

  <xsl:template match="split[SUM='*']"/>
  <xsl:template match="split[SUM='p']"/>
  <xsl:template match="split[SUM='i']"/>
  <xsl:template match="split">
    <!--
     id integer constraint split_pk primary key,
     accountid integer constraint split_account references account( id ),
     amount int default 0,
     reconcilestateid integer NOT NULL constraint split_state references reconcilestate( id ),
     reconciledate date,
     taxrate integer,
     memo varchar( 200 ),
     meta varchar( 500 )
    
    <xsl:variable name="memo" select="../../transaction/[../..transaction/TID=STID]/MEMO"/>
    -->
    <xsl:variable name="stid" select="STID"/>
    <xsl:variable name="named" select="../../namedtrans/namedtran[TID=$stid]"/>
    <xsl:choose>
      <xsl:when test="$named"/>
      <xsl:otherwise>
        <xsl:variable name="memo" select="../../transactions/transaction[TID=$stid]/MEMO"/>

        insert into split( id, accountid, amount, reconcilestateid, reconciledate, memo ) values (
        <xsl:value-of select="SID"/>,
        <xsl:value-of select="ACCTID"/>,
        <xsl:value-of select="SUM"/>,
        <xsl:value-of select="RECO"/>+1,
        <xsl:text>'</xsl:text><xsl:value-of select="RECODATE"/><xsl:text>'</xsl:text>,
        <xsl:choose>
          <xsl:when test="$memo!=''">
            <xsl:text>'</xsl:text><xsl:value-of select="$memo"/><xsl:text>'</xsl:text>
          </xsl:when>
          <xsl:otherwise>null</xsl:otherwise>
        </xsl:choose> );

        insert into trans_split( splitid, transactionid ) values (
        <xsl:value-of select="SID"/>,<xsl:value-of select="STID"/> );
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

</xsl:stylesheet>
