create table preference (
  param varchar( 50 ) constraint preference_pk primary key,
  value varchar( 100 )
);

create table transactiontype (
  id integer constraint transactiontype_pk primary key,
  description varchar( 50 ),
  iscalculable integer default 1 check( iscalculable=1 or iscalculable=0 )
);

create table journal (
  id integer constraint journal_pk primary key,
  name varchar( 50 ),
  description varchar( 100 )
);

create table accounttype (
  id integer constraint accounttype_pk primary key,
  description varchar( 50 ),
  isleftplus integer check( isleftplus=0 or isleftplus=1 )
);

create table account (
  id integer constraint account_pk primary key,
  parentid integer constraint account_parent references account( id ),
  name varchar( 50 ),
  openingbalance integer default 0,
  banknumber varchar( 100 ),
  notes varchar( 1000 ),
  taxrate integer,
  accounttypeid integer constraint account_type references accounttype( id ),
  meta varchar( 500 )
);

create table transactionnumberchoice (
   id integer constraint transnumchoice_pk primary key,
  description varchar( 50 ),
  isincrement integer check( isincrement=0 or isincrement=1 )
);

create table account_transnum(
  accountid integer constraint account_transnum_aid references account( id ),
  transnumid integer constraint account_transnum_tid references transactionnumberchoice( id )
);

create table transentry (
  id integer constraint transaction_pk primary key,
  typeid integer constraint transaction_type references transactiontype( id ),
  num varchar( 10 ),
  date date DEFAULT CURRENT_DATE,
  payee varchar( 100 ),
  journalid integer constraint transaction_journal references journal( id ),
  meta varchar( 500 )
);

create table reconcilestate (
  id integer constraint reconcilestate_pk primary key,
  description VARCHAR( 10 )
);

create table loan (
  id integer constraint loan_pk primary key,
  name varchar( 100 ) UNIQUE,
  principalid integer constraint loan_pid references account( id ),
  interestid integer constraint loan_pid references account( id ),
  periods integer,
  interest float,
  payment integer,
  amount integer
);

create table split (
  id integer constraint split_pk primary key,
  accountid integer constraint split_account references account( id ),
  amount int default 0,
  reconcilestateid integer NOT NULL constraint split_state references reconcilestate( id ),
  reconciledate date,
  taxrate integer,
  memo varchar( 200 ),
  meta varchar( 500 )
);

create table trans_split (
  transactionid integer constraint trans_split_trans_fk references transentry( id ),
  splitid integer constraint trans_split_split_fk references split( id )
);
create index transsplit on trans_split( transactionid );
create index splittrans on trans_split( splitid );

create table scheduletype (
  id integer primary key,
  description varchar( 30 )
);

create table schedule (
  id integer primary key,
  transactionid integer constraint schedule_trans_fk references transentry( id ),
  scheduletypeid integer constraint namedtrans_scheduletype references scheduletype( id ),
  name varchar( 500 ) unique,
  nextrun date
);

create table budgettimetype (
  id integer primary key,
  description varchar( 30 )
);

create table budget (
  id integer constraint budget_pk primary key,
  name varchar( 500 ),
  timetypeid int constraint budget_time_fk references budgettimetype( id ),
  otherlimit integer
);

create table budget_limit (
  budgetid integer constraint budgetlimit_b_fk references budget( id ),
  accountid integer constraint budgetlimit_a_fk references account( id ),
  blimit integer not null,
  includesubs integer check( includesubs=0 or includesubs=1 ),
  primary key( budgetid, accountid )
);


-- these values MUST match the enumerations in the DAL
insert into transactiontype( id, description, iscalculable ) values ( 1, 'Normal', 1 );
insert into transactiontype( id, description, iscalculable ) values ( 2, 'Void', 0 );
insert into transactiontype( id, description, iscalculable ) values ( 3, 'Template', 0 );
insert into transactiontype( id, description, iscalculable ) values ( 4, 'AccountDefault', 0 );

-- these values MUST match the enumerations in the DAL
insert into accounttype( id, description, isleftplus ) values ( 1, 'Asset', 0 );
insert into accounttype( id, description, isleftplus ) values ( 2, 'Liability', 1 );
insert into accounttype( id, description, isleftplus ) values ( 3, 'Equity', 0 );
insert into accounttype( id, description, isleftplus ) values ( 4, 'Revenue', 1 );
insert into accounttype( id, description, isleftplus ) values ( 5, 'Expense', 0 );

-- these values MUST match the enumerations in the DAL
insert into reconcilestate( id, description ) values ( 1, 'No' );
insert into reconcilestate( id, description ) values ( 2, 'Cleared' );
insert into reconcilestate( id, description ) values ( 3, 'Reconciled' );

-- these values MUST match the enumerations in the DAL
insert into budgettimetype( id, description ) values ( 1, 'Month to Date' );
insert into budgettimetype( id, description ) values ( 2, 'Running Month' );
insert into budgettimetype( id, description ) values ( 3, 'Year to Date' );
insert into budgettimetype( id, description ) values ( 4, 'Running Year' );

-- these values MUST match the enumerations in the DAL
insert into scheduletype( id, description ) values ( 0, 'Unscheduled' );
insert into scheduletype( id, description ) values ( 1, 'Monthly' );
insert into scheduletype( id, description ) values ( 2, 'Bi-Weekly' );
insert into scheduletype( id, description ) values ( 3, 'Last Day of Month' );
insert into scheduletype( id, description ) values ( 4, 'Bi-Monthly' );
insert into scheduletype( id, description ) values ( 5, 'Annually' );
insert into scheduletype( id, description ) values ( 6, 'Weekly' );
insert into scheduletype( id, description ) values ( 7, 'Daily' );

insert into journal( id, name ) values ( 1, 'General' );

