/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************
 * Created on July 12, 2009, 5:32 PM
 */

#include "Account.h"

Account::Account( int _id ) : IDableEntity( _id ) {
  parentid = 0;
  Money m( 0 );
  setBalance( m, Current );
  setBalance( m, Opening );
  setBalance( m, Reconciled );

  setTaxRate( 0 );
  setType( Asset );
}

Account::Account( const Account& orig ) : IDableEntity( orig ) {
  setName( orig.getName( ) );
  setType( orig.getType( ) );
  setBalance( orig.getBalance( Current ), Current );
  setBalance( orig.getBalance( Opening ), Opening );
  setBalance( orig.getBalance( Reconciled ), Reconciled );
  setNumber( orig.getNumber( ) );
  setTaxRate( orig.getTaxRate( ) );
  setNotes( orig.getNotes( ) );
  setParentId( orig.getParentId( ) );
}

Account& Account::operator=(const Account& orig ) {
  if ( this != &orig ) {
    setId( orig.getId( ) );
    setName( orig.getName( ) );
    setType( orig.getType( ) );
    setBalance( orig.getBalance( Current ), Current );
    setBalance( orig.getBalance( Opening ), Opening );
    setBalance( orig.getBalance( Reconciled ), Reconciled );
    setNumber( orig.getNumber( ) );
    setTaxRate( orig.getTaxRate( ) );
    setNotes( orig.getNotes( ) );
    setParentId( orig.getParentId( ) );
  }

  return *this;
}

void Account::setType( const AccountType& t ) {
  type = t;
}

AccountType Account::getType( ) const {
  return type;
}

void Account::setName( const QString& n ) {
  name = n;
}

QString Account::getName( ) const {
  return name;
}

void Account::setBalance( const Money& m, AccountBalance t ) {
  switch ( t ) {
  case Current:
    cbal = m;
    break;
  case Opening:
    obal = m;
    break;
  case Reconciled:
    rbal = m;
    break;
  }
}

Money Account::getBalance( AccountBalance t ) const {
  Money bal;
  if ( Opening == t ) {
    bal = obal;
  }
  else if ( Reconciled == t ) {
    bal = rbal;
  }
  else {
    bal = cbal;
  }

  return bal;
}

void Account::setNumber( const QString& n ) {
  number = n;
}

QString Account::getNumber( ) const {
  return number;
}

void Account::setNotes( const QString& n ) {
  notes = n;
}

QString Account::getNotes( ) const {
  return notes;
}

void Account::setTaxRate( int t ) {
  taxrate = t;
}

int Account::getTaxRate( ) const {
  return taxrate;
}

void Account::setParentId( int i ) {
  parentid = i;
}

int Account::getParentId( ) const {
  return parentid;
}

bool Account::isRightPlus( ) const {
  return ( Liability == type || Equity == type || Revenue == type );
}

bool Account::operator<( const Account& a ) const{
  return getId() < a.getId();
}
