#include "Budget.h"
#include "utility"
#include "Account.h"

Budget::Budget( int id ) : IDableEntity( id ){
  timer = MonthToDate;
}

Budget::Budget( const Budget& orig ) : IDableEntity( orig.getId() ),
  items( orig.items ), name( orig.getName() ), timer( orig.timer ),
  otherlimit( orig.otherlimit ) {
}

Budget& Budget::operator=( const Budget& orig ){
  if( this != &orig ){
    IDableEntity::operator=( orig );
    name = orig.getName();
    timer = orig.timer;
    items = orig.items;
    otherlimit = orig.otherlimit;
  }

  return *this;
}

const std::pair<Money, bool>& Budget::operator[]( const Account& a ) const throw ( MapperException ) {
  return operator[]( a.getId() );
}

const std::pair<Money, bool>& Budget::operator[]( int aid ) const throw ( MapperException ){
  if( items.count( aid ) > 0 ){
    return items.at( aid );
  }
  throw MapperException( "Account not present" );

}

void Budget::addBudgetItem( const Account& aid, const Money& limit,
                            bool includesubs ){
  addBudgetItem( aid.getId(), limit, includesubs );
}

void Budget::removeBudgetItem( const Account& aid ){
  removeBudgetItem( aid.getId() );
}

void Budget::addBudgetItem( int aid, const Money& limit, bool andsubs ){
  items.erase( aid );
  items.insert( std::make_pair( aid, std::make_pair( limit, andsubs ) ) );
}

void Budget::removeBudgetItem( int aid ){
  items.erase( aid );
}

void Budget::setName( const QString& n ){
  name = n;
}

const QString& Budget::getName() const{
  return name;
}

void Budget::setTime( const BudgetTime& bt ){
  timer = bt;
}

const BudgetTime& Budget::getTime() const {
  return timer;
}

bool Budget::contains( const Account& a ) const{
  return contains( a.getId() );
}

bool Budget::contains( int aid ) const{
  return ( items.count(aid ) > 0 );
}

void Budget::setOtherLimit( const Money& m ){
  otherlimit = m;
}

const Money& Budget::getOtherLimit() const {
  return otherlimit;
}
