#ifndef BUDGET_H
#define BUDGET_H

#include "IDableEntity.h"
#include "MapperException.h"
#include "Constants.h"
#include "Money.h"

#include <map>

class Account;

class Budget : public IDableEntity {
public:
  Budget( int id =IDableEntity::NOID );
  Budget( const Budget& orig );
  virtual ~Budget(){}

  Budget& operator=( const Budget& orig );
  const std::pair<Money, bool>& operator[]( const Account& ) const throw ( MapperException );
  const std::pair<Money, bool>& operator[]( int ) const throw ( MapperException );

  void addBudgetItem( const Account& aid, const Money& limit, bool includesubs = false );
  void removeBudgetItem( const Account& aid );

  void addBudgetItem( int aid, const Money& limit, bool andsubs = false );
  void removeBudgetItem( int aid );

  void setName( const QString& name );
  const QString& getName() const;

  void setTime( const BudgetTime& bt );
  const BudgetTime& getTime() const;

  void setOtherLimit( const Money& m );
  const Money& getOtherLimit() const;

  const std::map<int, std::pair<Money, bool> >& getLimits() const {
    return items;
  }

  bool contains( const Account& ) const;
  bool contains( int aid ) const;

private:
  std::map<int, std::pair<Money, bool> > items;
  QString name;
  BudgetTime timer;
  Money otherlimit;
};

#endif // BUDGET_H
