/*
 * File:   FullTransaction.cpp
 * Author: ryan
 */

#include "FullTransaction.h"
#include "Account.h"
#include "Logger.h"

FullTransaction::FullTransaction( const Transaction& t, const QList<Split>& s )
  : trans( t ), splits( s ) {
}

FullTransaction::FullTransaction( const FullTransaction& orig )
: trans( orig.trans ), splits( orig.splits ) {
}

FullTransaction & FullTransaction::operator=(const FullTransaction& orig ) {
  if ( this != &orig ) {
    trans = orig.trans;
    setSplits( orig.splits );
  }

  return *this;
}

bool FullTransaction::operator==( const FullTransaction& orig ) const {
  if ( orig.getTransaction( ).getId( ) == trans.getId( ) ) {
    // WARNING: probably want to check splits as well...
    return true;
  }
  return false;
}

bool FullTransaction::operator!=( const FullTransaction& orig ) const {
  return !( *this == orig );
}

bool FullTransaction::isSplit() const {
  return ( splits.count() > 2 );
}

int FullTransaction::otherAccountId( const Account& account ) const {
  return otherAccountId( account.getId() );
}

int FullTransaction::otherAccountId( int aid ) const {
  int ret = IDableEntity::NOID;
  if( !isSplit() ){
    for( const auto& s : splits ) {
      const int otherid = s.getAccountId();
      if ( otherid != aid ) {
        ret = otherid;
      }
    }
  }
  return ret;
}


const Split& FullTransaction::operator[]( const Account& a ) const
throw ( MapperException ) {
  return operator[]( a.getId( ) );
}

const Split& FullTransaction::operator[]( int aid ) const throw ( MapperException ) {

  for( const auto& s : splits ) {
    if ( s.getAccountId( ) == aid ) {
      return s;
    }
  }
  throw MapperException( "Account not present" );
}

void FullTransaction::add( const Split& s ) {
  splits.append( s );
}

void FullTransaction::setSplits( const QList<Split>& s ) {
  splits.clear( );
  splits.append( s );
}

const Transaction& FullTransaction::getTransaction( ) const {
  return trans;
}

const QList<Split>& FullTransaction::getSplits( ) const {
  return splits;
}

void FullTransaction::reset( const Transaction& t, const QList<Split>& s ){
  trans = t;
  setSplits( s );
}

int FullTransaction::update( const Split& s ) {
  for ( int i = 0; i < splits.count( ); i++ ) {
    if ( s == splits[i] ) {
      splits[i] = s;
      return i;
    }
  }
  return -1;
}

Money FullTransaction::balanceMinus(const Split &s) const{
  Money bal;
  foreach( const Split& other, splits ){
    if( other != s ){
      bal += other.getValue();
    }
  }
  return bal;
}

bool FullTransaction::hasSplitForAccount( int aid ) const {

  for( const auto & s : splits ) {
    if ( s.getAccountId( ) == aid ) {
      return true;
    }
  }

  return false;
}

bool FullTransaction::hasSplitForAccount( const Account& a ) const {
  return hasSplitForAccount( a.getId( ) );
}

bool FullTransaction::isNull() const {
  return ( Transaction::NOID == trans.getId() );
}
