/*
 * File:   FullTransaction.h
 * Author: ryan
 */

#ifndef FULLTRANSACTION_H
#define	FULLTRANSACTION_H

#include <QList>

#include "Transaction.h"
#include "TransactionSchedule.h"
#include "Split.h"
#include "MapperException.h"

class Account;

class FullTransaction {
public:
  FullTransaction( const Transaction& t = Transaction( ),
      const QList<Split>& s = QList<Split>( ) );
  FullTransaction( const FullTransaction& orig );
  FullTransaction & operator=( const FullTransaction& orig );
  virtual ~FullTransaction( ) {}

  bool operator==(const FullTransaction& orig ) const;
  bool operator!=(const FullTransaction& orig ) const;
  const Split& operator[]( const Account& a ) const throw ( MapperException );
  const Split& operator[]( int aid ) const throw ( MapperException );
  bool hasSplitForAccount( int aid ) const;
  bool hasSplitForAccount( const Account& ) const;
  bool isSplit() const;
  /**
    * For split transactions, returns NOID. Else, returns the remaining
    * account id when the given account id is removed.
    */
  int otherAccountId( const Account& ) const;
  int otherAccountId( int aid ) const;

  const Transaction& getTransaction( ) const;
  const QList<Split>& getSplits( ) const;

  void reset( const Transaction& t, const QList<Split>& s = QList<Split>() );
  void add( const Split& s );
  void setSplits( const QList<Split>& s );
  /**
   * Updates a split
   * @param the split to update
   * @return the index of the updated split, or -1 if the split doesn't exist
   */
  int update( const Split& );

  Money balanceMinus( const Split& s ) const;
  bool isNull() const;

private:
  Transaction trans;
  QList<Split> splits;
};

#endif	/* FULLTRANSACTION_H */

