
#include "Loan.h"
#include "Logger.h"

#include <math.h>

Loan::Loan( int _id ) : IDableEntity( _id ), periods( 0 ), payment( 0 ),
  amount( 0 ), periodRate( 0 ), principalAccountId( IDableEntity::NOID ),
  interestAccountId( IDableEntity::NOID ){
}

Loan::Loan( const Loan& orig ) : IDableEntity( orig ), name( orig.name ),
  periods( orig.periods ), payment( orig.payment ), amount( orig.amount ),
  periodRate( orig.periodRate ), principalAccountId( orig.principalAccountId ),
  interestAccountId( orig.interestAccountId ){
}

Loan& Loan::operator=( const Loan& orig ){
  if( this != &orig ){
    IDableEntity::operator=( orig );
    periods = orig.periods;
    payment = orig.payment;
    amount = orig.amount;
    periodRate = orig.periodRate;
    principalAccountId = orig.principalAccountId;
    interestAccountId = orig.interestAccountId;
    name = orig.name;
  }

  return *this;
}

void Loan::setName( const QString& str ){
  name = str;
}

const QString& Loan::getName() const {
  return name;
}

void Loan::setPeriods( int p ){
  periods = p;
}

int Loan::getPeriods() const{
  return periods;
}

void Loan::setPayment( const Money& m ){
  payment = m;
}

const Money& Loan::getPayment() const{
  return payment;
}

void Loan::setAmount( const Money& m ){
  amount = m;
}

const Money& Loan::getAmount() const{
  return amount;
}

void Loan::setYearlyInterestRate( double rate ){
  periodRate = rate / 12;
}

double Loan::getYearlyInterestRate() const{
  return getPeriodInterestRate() * 12;
}

double Loan::getPeriodInterestRate() const{
  return periodRate;
}

void Loan::setPrincipalAccountId( int aid ){
  principalAccountId = aid;
}

int Loan::getPrincipalAccountId() const{
  return principalAccountId;
}

void Loan::setInterestAccountId( int aid ){
  interestAccountId = aid;
}

int Loan::getInterestAccountId() const{
  return interestAccountId;
}

Money Loan::getPrincipalPayment( const Money &loanBalance ) const {
  return payment - getInterestPayment( loanBalance );
}

Money Loan::getInterestPayment( const Money &loanBalance ) const {
  double newamt = loanBalance.getValue() * periodRate;
  double newamt2 = ceil( newamt );
  int val = static_cast<int>( newamt2 );
  return Money( val );
}
