/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************/

#include "QHacc.h"

#include "AccountMapper.h"
#include "SplitMapper.h"
#include "TransactionMapper.h"
#include "JournalMapper.h"
#include "LoanMapper.h"
#include "BudgetMapper.h"
#include "TransactionUtils.h"
#include "AccountUtils.h"
#include "EngineNotifier.h"
#include "LoanUtils.h"
#include "BudgetUtils.h"
#include "TransactionSchedule.h"

#include <QSettings>

QHacc * QHacc::instance = nullptr;

QHacc::~QHacc(){}

QHacc::QHacc( QSqlDatabase& conn ) :
  acctMapper( new AccountMapper( conn, this ) ),
  splitMapper( new SplitMapper( conn, this ) ),
  transMapper( new TransactionMapper( conn, this ) ),
  journalMapper( new JournalMapper( conn, this ) ),
  loanMapper( new LoanMapper( conn, this ) ),
  budgetMapper( new BudgetMapper( conn, this ) ),
  tutils( new TransactionUtils( conn, this ) ),
  autils( new AccountUtils( conn, this ) ),
  notifier( new EngineNotifier( ) ),
  lutils( new LoanUtils( conn, this ) ),
  butils( new BudgetUtils( conn, this ) ){}

void QHacc::init( QSqlDatabase& conn ) {
  instance = new QHacc( conn );
  instance->executeSchedulesSince( QDate::currentDate() );
}

QHacc * QHacc::getInstance( ) {
  return instance;
}

void QHacc::cleanup( ) {
  delete instance;
}

AccountMapper& QHacc::getAccountMapper( ) const {
  return *acctMapper;
}

SplitMapper& QHacc::getSplitMapper( ) const {
  return *splitMapper;
}

TransactionMapper& QHacc::getTransactionMapper( ) const {
  return *transMapper;
}

JournalMapper& QHacc::getJournalMapper( ) const {
  return *journalMapper;
}

LoanMapper& QHacc::getLoanMapper() const {
  return *loanMapper;
}

BudgetMapper& QHacc::getBudgetMapper( ) const {
  return *budgetMapper;
}

QList<FullTransaction> QHacc::getTransactions( const Account & a,
    const Journal& j ) const {
  return tutils->getTransactions( a, j );
}

void QHacc::save( FullTransaction& ft ) {
  tutils->save( ft );
}

void QHacc::update( const FullTransaction& older,
    FullTransaction& newer ) {
  tutils->update( older, newer );
}

EngineNotifier * QHacc::getNotifier( ) const {
  return notifier.get();
}

AccountUtils& QHacc::getAccountUtils( ) const {
  return *autils;
}

TransactionUtils& QHacc::getTransactionUtils( ) const {
  return *tutils;
}

LoanUtils& QHacc::getLoanUtils( ) const {
  return *lutils;
}

BudgetUtils& QHacc::getBudgetUtils() const{
  return *butils;
}

void QHacc::executeSchedulesSince( const QDate& lastrun ){
  for( TransactionSchedule ts : transMapper->getSchedules() ){
    while( ts.isDue( lastrun ) ){
      tutils->execute( ts );
    }
  }
}
