/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************/


#ifndef _QHACC_H
#define	_QHACC_H

#include <QList>
#include <QSqlDatabase>
#include <memory>

#include "Transaction.h"
#include "Split.h"
#include "FullTransaction.h"

class Account;
class Journal;
class AccountMapper;
class SplitMapper;
class TransactionMapper;
class JournalMapper;
class LoanMapper;
class BudgetMapper;
class TransactionUtils;
class AccountUtils;
class EngineNotifier;
class LoanUtils;
class BudgetUtils;

class QHacc {
public:
  virtual ~QHacc( );

  AccountMapper& getAccountMapper( ) const;
  SplitMapper& getSplitMapper( ) const;
  TransactionMapper& getTransactionMapper( ) const;
  JournalMapper& getJournalMapper( ) const;
  LoanMapper& getLoanMapper( ) const;
  BudgetMapper& getBudgetMapper( ) const;

  QList<FullTransaction> getTransactions( const Account & a,
      const Journal& j ) const;

  void save( FullTransaction& ft );
  void update( const FullTransaction& older, FullTransaction& newer );

  EngineNotifier * getNotifier( ) const;
  AccountUtils& getAccountUtils() const;
  TransactionUtils& getTransactionUtils() const;
  LoanUtils& getLoanUtils() const;
  BudgetUtils& getBudgetUtils() const;

  static void init( QSqlDatabase& conn );
  static QHacc * getInstance( );
  static void cleanup( );

private:
  QHacc( QSqlDatabase& conn );
  void executeSchedulesSince( const QDate& lastrun );

  std::unique_ptr<AccountMapper> acctMapper;
  std::unique_ptr<SplitMapper> splitMapper;
  std::unique_ptr<TransactionMapper> transMapper;
  std::unique_ptr<JournalMapper> journalMapper;
  std::unique_ptr<LoanMapper> loanMapper;
  std::unique_ptr<BudgetMapper> budgetMapper;
  std::unique_ptr<TransactionUtils> tutils;
  std::unique_ptr<AccountUtils> autils;
  std::unique_ptr<EngineNotifier> notifier;
  std::unique_ptr<LoanUtils> lutils;
  std::unique_ptr<BudgetUtils> butils;

  static QHacc * instance;
};

#endif	/* _QHACC_H */

