/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************
 * Created on July 14, 2009, 8:22 PM
 */

#include "Split.h"
#include "Account.h"

Split::Split( int id ) : IDableEntity( id ), accountid( NOID ),
  amount( Money( 0 ) ), reco( No ), taxrate( 0 ) {}

Split::Split( const Split& orig ) : IDableEntity( orig ),
  accountid( orig.accountid ), amount( orig.amount ), reco( orig.reco ),
  recodate( orig.recodate ), taxrate( orig.taxrate ), memo( orig.memo ) {
}

Split& Split::operator=(const Split& orig ) {
  if ( this != &orig ) {
    setId( orig.getId( ) );
    setAccountId( orig.getAccountId( ) );
    setValue( orig.getValue( ) );
    setReconciled( orig.getReconciled( ), orig.recodate );
    setTaxRate( orig.getTaxRate( ) );
    setMemo( orig.getMemo( ) );
  }
  return *this;
}

Split & Split::setValue( const Money& m ) {
  amount = m;
  return *this;
}

Money Split::getValue( ) const {
  return amount;
}

Split & Split::setReconciled( const ReconcileState& rs, const QDate& d ) {
  reco = rs;
  recodate = d;
  return *this;
}

ReconcileState Split::getReconciled( ) const {
  return reco;
}

QDate Split::getReconcileDate( ) const {
  return recodate;
}

Split & Split::setTaxRate( int tr ) {
  taxrate = tr;
  return *this;
}

int Split::getTaxRate( ) const {
  return taxrate;
}

Split & Split::setMemo( const QString& m ) {
  memo = m;
  return *this;
}

QString Split::getMemo( ) const {
  return memo;
}

Split & Split::setAccountId( int aid ) {
  accountid = aid;
  return *this;
}

int Split::getAccountId( ) const {
  return accountid;
}

bool Split::hasAccount( ) const {
  return ( NOID != accountid );
}

QString Split::toString( ) const {
  return QString( "Split %1, account %2: %3" ).
      arg( getId( ) ).arg( getAccountId( ) ).arg( getValue( ).toString( ) );
}
