/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************
 * Created on July 14, 2009, 8:21 PM
 */

#include "Logger.h"


#include "Transaction.h"
#include "Journal.h"

Transaction::Transaction( int id ) : IDableEntity( id ), type( Normal ) {
}

Transaction::Transaction( const Transaction& orig ) : IDableEntity( orig ),
  journalid( orig.journalid ), date( orig.date ), payee( orig.payee ),
  memo( orig.memo ), number( orig.number ), type( orig.type ) {
}

Transaction& Transaction::operator=( const Transaction& orig ) {
  if ( this != &orig ) {
    setId( orig.getId( ) );
    setJournalId( orig.getJournalId( ) );
    setDate( orig.getDate( ) );
    setPayee( orig.getPayee( ) );
    setMemo( orig.getMemo( ) );
    setType( orig.getType( ) );
    setNumber( orig.getNumber( ) );
  }
  return *this;
}

void Transaction::setDate( const QDate& d ) {
  date = d;
}

const QDate &Transaction::getDate( ) const {
  return date;
}

void Transaction::setJournalId( int id ) {
  journalid = id;
}

int Transaction::getJournalId( ) const {
  return journalid;
}

void Transaction::setMemo( const QString& m ) {
  memo = m;
}

const QString &Transaction::getMemo( ) const {
  return memo;
}

void Transaction::setNumber( const QString& s ) {
  number = s;
}

const QString& Transaction::getNumber( ) const {
  return number;
}

void Transaction::setPayee( const QString& s ) {
  payee = s;
}

const QString& Transaction::getPayee( ) const {
  return payee;
}

void Transaction::setType( const TransactionType& tt ) {
  type = tt;
}

TransactionType Transaction::getType( ) const {
  return type;
}

bool Transaction::operator==( const Transaction& orig ) const {
  return orig.getId( ) == getId( );
}

uint qHash( const Transaction& t ) {
  return t.getId( )+(int) t.getType( );
}
