/*
 * File:   TransactionSchedule.cpp
 * Author: ryan
 *
 * Created on November 27, 2011, 9:41 PM
 */

#include "TransactionSchedule.h"

TransactionSchedule::TransactionSchedule( int id, ScheduleType typ, int tid,
  const QString &n, const QDate &next ) : IDableEntity( id ), transid( tid ),
  name( n ), type( next.isValid() ? typ : Unscheduled ), nextrun( next ){}


TransactionSchedule::TransactionSchedule( const TransactionSchedule& orig )
  : IDableEntity( orig ), transid( orig.transid ), name( orig.name ),
  type( orig.type ), nextrun( orig.nextrun ) {}

TransactionSchedule& TransactionSchedule::operator=( const TransactionSchedule& o ) {
  if ( this != &o ) {
    IDableEntity::operator=( o );
    setName( o.getName( ) );
    transid = o.transid;
    setNextRun( o.getNextRun(), o.getType() );
  }
  return *this;
}

bool TransactionSchedule::isNull( ) const {
  return ( transid == IDableEntity::NOID );
}

ScheduleType TransactionSchedule::getType() const {
  return ( nextrun.isValid() ? type : Unscheduled );
}

void TransactionSchedule::setNextRun( const QDate& d, ScheduleType ty ){
  nextrun = d;
  type = ( nextrun.isValid() ? ty : Unscheduled );
}

const QDate& TransactionSchedule::getNextRun() const {
  return nextrun;
}

void TransactionSchedule::setName( const QString& s ){
  name = s;
}

const QString& TransactionSchedule::getName() const {
  return name;
}

bool TransactionSchedule::isDue( const QDate& since ) const {
  if( Unscheduled == getType() ){
    return false;
  }

  return ( since.daysTo( nextrun ) <= 0 );
}
