/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************
 * Created on July 12, 2009, 5:25 PM
 */

#ifndef _TRANSACTIONUTILS_H
#define	_TRANSACTIONUTILS_H

#include <QList>
#include <QDate>
#include <QStringList>

#include "MapperException.h"
#include "FullTransaction.h"
#include "Journal.h"

class QSqlDatabase;
class QHacc;
class Account;
class Loan;
class TransactionSchedule;

class TransactionUtils {
public:
  TransactionUtils( QSqlDatabase& conn, QHacc * e );
  TransactionUtils( const TransactionUtils& orig );
  virtual ~TransactionUtils( ){}

  QList<FullTransaction> getTransactions( const Account& a,
      const Journal& j = Journal( ), const QDate& end = QDate( ),
      const QDate& start = QDate( ) ) throw ( MapperException );
  QList<FullTransaction> getTransactions( const QString& payee );

  void save( FullTransaction& ft );
  void update( const FullTransaction& older, FullTransaction& newer );
  void remove( const FullTransaction& );

  FullTransaction getTransaction( int tid ) throw ( MapperException );
  FullTransaction getTransaction( const TransactionSchedule& ts ) throw ( MapperException );
  QStringList getPayees( const Account& );
  QStringList getPayees( int aid );
  QStringList getMostPopularPayees( int fromAid, int toAid, int topx = 10 );

  void execute( TransactionSchedule& ts );

private:
  TransactionUtils( );

  void linkSplitToTrans( const Split& s, const Transaction& t ) throw ( MapperException );
  void step( TransactionSchedule& ts );

  /**
   * Removes duplicate or zero-sum splits
   * @param splits
   * @return
   */
  QList<Split> simplifySplits( QList<Split> splits ) const;

  QSqlDatabase * conn;
  QHacc * engine;
};

#endif	/* _TRANSACTIONUTILS_H */

