/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************/

#include <QSqlQuery>

#include "EngineNotifier.h"
#include <QSqlError>
#include <QVariant>

#include "JournalMapper.h"
#include "Account.h"
#include "QHacc.h"
#include "Logger.h"

JournalMapper::JournalMapper( const QSqlDatabase& conn, const QHacc * e )
: DBMapper<Journal>( conn, e ) {
}

JournalMapper::JournalMapper( const JournalMapper& orig ) : DBMapper<Journal>( orig ) {
}

Journal JournalMapper::getById( int id ) throw ( MapperException ) {
  if ( cache.contains( id ) ) {
    return Journal( *( cache.object( id ) ) );
  }
  else {
    Journal * journal = new Journal( id );
    QSqlQuery query;
    query.setForwardOnly( true );
    query.prepare( "SELECT name FROM journal WHERE id=?" );
    query.bindValue( 0, QVariant( id ) );

    if ( query.exec( ) && query.next( ) ) {
      journal->setName( query.value( 0 ).toString( ) );
      cache.insert( id, journal );
      return Journal( *journal );
    }
    else {
      Logger::error( query.lastError( ).text( ) );
      throw MapperException( query.lastError( ).text( ) );
    }
  }
}

void JournalMapper::save( Journal& t ) throw ( MapperException ) {
  QSqlQuery query;
  query.setForwardOnly( true );
  query.prepare( "INSERT INTO journal( name ) VALUES ( ? )" );
  query.bindValue( 0, t.getName( ) );

  if ( query.exec( ) ) {
    t.setId( query.lastInsertId( ).toInt( ) );
    engine->getNotifier( )->journal( t );
  }
  else {
    Logger::error( query.lastError( ).text( ) );
    throw MapperException( query.lastError( ).text( ) );
  }
}

void JournalMapper::update( Journal & t ) throw ( MapperException ) {
  Journal j( getById( t.getId( ) ) );
  cache.remove( t.getId( ) );

  QSqlQuery query;
  query.setForwardOnly( true );
  query.prepare( "UPDATE journal set name=? WHERE id=?" );
  query.bindValue( 0, t.getName( ) );
  query.bindValue( 1, t.getId( ) );

  if ( query.exec( ) ) {
    engine->getNotifier( )->journal( j, t );
  }
  else {
    Logger::error( query.lastError( ).text( ) );
    throw MapperException( query.lastError( ).text( ) );
  }
}

void JournalMapper::remove( const Journal& t ) throw ( MapperException ) {
  cache.remove( t.getId( ) );
  QSqlQuery query;
  query.setForwardOnly( true );
  query.prepare( "DELETE FROM journal WHERE id=?" );
  query.bindValue( 0, t.getId( ) );
  if ( query.exec( ) ) {
    engine->getNotifier( )->journal( t.getId( ) );
  }
  else {
    Logger::error( query.lastError( ).text( ) );
    throw MapperException( query.lastError( ).text( ) );
  }
}

QList<Journal> JournalMapper::getAll( ) throw ( MapperException ) {
  QList<Journal> list;
  QSqlQuery query;
  query.setForwardOnly( true );
  query.prepare( "SELECT id FROM journal" );

  if ( !query.exec( ) ) {
    Logger::error( query.lastError( ).text( ) );
    throw MapperException( query.lastError( ).text( ) );
  }

  while ( query.next( ) ) {
    int id = query.value( 0 ).toInt( );
    list.append( getById( id ) );
  }

  return list;
}
