/*
 * File:   AccountEditor.cpp
 * Author: ryan
 */

#include <QComboBox>
#include <QLineEdit>
#include <QSortFilterProxyModel>
#include <QKeyEvent>

#include "AccountDelegate.h"
#include "AccountMapper.h"
#include "AccountDialog.h"
#include "QHacc.h"
#include "AccountUtils.h"
#include "Logger.h"
#include "AccountComboBox.h"

AccountDelegate::AccountDelegate( bool allowCreate, QObject *p )
: QStyledItemDelegate( p ), allowNew( allowCreate ) {
}

QWidget * AccountDelegate::createEditor( QWidget *parent,
    const QStyleOptionViewItem &, const QModelIndex & ) const {
  return new AccountComboBox( parent, allowNew );
}

void AccountDelegate::setEditorData( QWidget * editor,
    const QModelIndex &index ) const {
  int aid = index.data( Qt::UserRole ).toInt( );

  QComboBox * combo = static_cast<QComboBox*> ( editor );
  combo->setCurrentIndex( combo->findData( aid ) );
}

void AccountDelegate::setModelData( QWidget *editor, QAbstractItemModel *model,
    const QModelIndex &index ) const {
  QComboBox * combo = static_cast<QComboBox*> ( editor );
  int pos = combo->currentIndex( );
  int value = combo->itemData( pos, Qt::UserRole ).toInt( );

  if ( IDableEntity::NOID == value ) {
    Logger::debug( "new account option selected" );
    Account newacct;
    AccountDialog dlg( newacct );
    dlg.exec();
    value = dlg.getCurrent().getId();
  }

  if( IDableEntity::NOID != value ){
    model->setData( index, value, Qt::EditRole );
  }
}

void AccountDelegate::updateEditorGeometry( QWidget *editor,
    const QStyleOptionViewItem &option, const QModelIndex & ) const {
  editor->setGeometry( option.rect );
}

bool AccountDelegate::eventFilter(QObject * editor, QEvent *event){
  if( QEvent::KeyPress == event->type() ){
    QKeyEvent * keyEvent = static_cast<QKeyEvent*>( event );
    if ( Qt::Key_Return == keyEvent->key( ) ) {
      Logger::debug( "ate key event!" );
      emit commitData( (QWidget *) editor );
      emit closeEditor( (QWidget *) editor, QAbstractItemDelegate::EditNextItem );
      return true;
    }
  }

  return QStyledItemDelegate::eventFilter(editor, event);
}
