/*
 * File:   AccountDialog.cpp
 * Author: ryan
 *
 * Created on November 29, 2011, 8:22 PM
 */

#include "AccountDialog.h"
#include "Constants.h"
#include "QHacc.h"
#include "AccountMapper.h"

#include <QButtonGroup>
#include <QPushButton>
#include <QRadioButton>

AccountDialog::AccountDialog( const Account& a, QWidget * p ) : QDialog( p ), current( a ) {
  widget.setupUi( this );

  QVBoxLayout * vbox = new QVBoxLayout;
  QRadioButton * asset = new QRadioButton( tr( "Asset" ) );
  QRadioButton * equity = new QRadioButton( tr( "Equity" ) );
  QRadioButton * liability = new QRadioButton( tr( "Liability" ) );
  QRadioButton * revenue = new QRadioButton( tr( "Revenue" ) );
  QRadioButton * expense = new QRadioButton( tr( "Expense" ) );
  vbox->addWidget( asset );
  vbox->addWidget( equity );
  vbox->addWidget( liability );
  vbox->addWidget( revenue );
  vbox->addWidget( expense );
  vbox->addStretch( 1 );
  widget.groupBox->setLayout( vbox );

  grp.reset( new QButtonGroup( ) );
  grp->addButton( asset, Asset );
  grp->addButton( equity, Equity );
  grp->addButton( liability, Liability );
  grp->addButton( revenue, Revenue );
  grp->addButton( expense, Expense );

  connect( grp.get(), SIGNAL( buttonClicked( int ) ),
      SLOT( changeAccountType( int ) ) );
  QPushButton * btn = widget.buttonBox->button( QDialogButtonBox::Ok );
  connect( btn, SIGNAL( clicked( ) ), this, SLOT( save( ) ) );

  widget.balance->setText( a.getBalance( Opening ).toString( ) );
  widget.name->setText( a.getName( ) );
  widget.notes->setPlainText( a.getNotes( ) );
  widget.number->setText( a.getNumber( ) );
  QAbstractButton * rbtn = grp->button( a.getType( ) );
  rbtn->setChecked( true );
}

AccountDialog::~AccountDialog( ) {}

void AccountDialog::changeAccountType( int id ) {
  current.setType( static_cast<AccountType> ( id ) );
}

void AccountDialog::save( ) {
  current.setBalance( Money::getMoney( widget.balance->text( ) ), Opening );
  current.setName( widget.name->text( ) );
  current.setNotes( widget.notes->toPlainText( ) );
  current.setNumber( widget.number->text( ) );

  if ( IDableEntity::NOID == current.getId( ) ) {
    QHacc::getInstance( )->getAccountMapper( ).save( current );
  }
  else {
    QHacc::getInstance( )->getAccountMapper( ).update( current );
  }
}

const Account& AccountDialog::getCurrent() const {
  return current;
}
