/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************/


#ifndef ACCOUNTVIEWER_H
#define	ACCOUNTVIEWER_H

#include "TransactionViewer.h"
#include "FullTransaction.h"

#include <QHash>
#include <QTreeView>

class Split;
class QSortFilterProxyModel;
class QMouseEvent;

class AccountViewer : public TransactionViewer {
  Q_OBJECT
public:
  AccountViewer( const Journal& j, QWidget * p = 0 );
  virtual ~AccountViewer( ){}
  void showAllSplits( bool showit = true );
  static const int ISTRANSROLE;

  void keyPressEvent( QKeyEvent *event );

public slots:
  virtual void showMenu( const QPoint & pos );
  virtual void insertMem( const QString& str );
  virtual void insertLoan( int loanid );
  virtual void transactionChanged( QTreeWidgetItem *, int );
  virtual void updateTrans( const FullTransaction&, const FullTransaction& );
  virtual void updateSplit( const Split&, const Split& );
  virtual void addTrans( const FullTransaction& );
  virtual void memorize( int tid );
  virtual void pasteTrans();
  virtual void copyTrans();

signals:
  void editTrans( const FullTransaction& ft );

protected:
  static const int TRANSROLE;
  virtual void mouseReleaseEvent( QMouseEvent * event );
  virtual QTreeWidgetItem * addSplit( const Split&, QTreeWidgetItem * transitem );

  /**
   * Should this view include this transaction?
   * @param the transaction
   * @return true, if this transaction should be included in this widget
   */
  virtual bool shouldIncludeTrans( const FullTransaction& );
  void setJournal( const Journal& );
  virtual void cleanup();
  void copy( int tid );


  QHash<int, QTreeWidgetItem *> tidItemLkp;
  QHash<int, QTreeWidgetItem *> sidItemLkp;
  bool showAll;
  QPoint rightclickpos;
  FullTransaction copytrans;
};

#endif	/* ACCOUNTVIEWER_H */

