/*
 * File:   BoolDelegate.cpp
 * Author: ryan
 *
 * Created on November 30, 2011, 2:16 PM
 */

#include <QCheckBox>
#include <QPainter>
#include <QApplication>

#include "BoolDelegate.h"
#include "Constants.h"

BoolDelegate::BoolDelegate( bool tri, QObject * p ) : QStyledItemDelegate( p ),
  tristate( tri ) {
}

QWidget * BoolDelegate::createEditor( QWidget * parent,
    const QStyleOptionViewItem &, const QModelIndex & ) const {
  QCheckBox * qcb = new QCheckBox( parent );
  qcb->setTristate( tristate );
  return qcb;
}

void BoolDelegate::setEditorData( QWidget * editor,
    const QModelIndex &index ) const {
  QCheckBox * qcb = static_cast<QCheckBox *> ( editor );
  bool checked = index.data( Qt::UserRole ).toBool( );
  qcb->setCheckState( checked ? Qt::Checked : Qt::Unchecked );
}

void BoolDelegate::setModelData( QWidget * editor, QAbstractItemModel * model,
    const QModelIndex &index ) const {
  QCheckBox * qcb = static_cast<QCheckBox *> ( editor );
  model->setData( index, qcb->isChecked(), Qt::EditRole );
}

void BoolDelegate::updateEditorGeometry( QWidget *editor,
    const QStyleOptionViewItem &option, const QModelIndex & ) const {
  editor->setGeometry( option.rect );
}

void BoolDelegate::paint( QPainter * painter, const QStyleOptionViewItem& opt,
              const QModelIndex& index ) const {
  painter->save();

  bool data = index.data().toBool();
  QStyleOptionButton checkboxstyle;
  checkboxstyle.rect = opt.rect;
  checkboxstyle.state = ( data ? QStyle::State_On : QStyle::State_Off ) |
      QStyle::State_Enabled;
  QApplication::style()->drawControl( QStyle::CE_CheckBox, &checkboxstyle,
                                      painter );
  painter->restore();
}
