#include "BudgetDialog.h"

#include "QHacc.h"
#include "BudgetMapper.h"
#include "Logger.h"
#include "AccountMapper.h"
#include "AccountUtils.h"
#include "BudgetMapper.h"
#include "BudgetUtils.h"
#include "BudgetLimitRowModel.h"
#include "AccountDelegate.h"
#include "BoolDelegate.h"
#include "MoneyDelegate.h"

#include <QTableWidgetItem>
#include <QSettings>

const QString BudgetDialog::settingsKey( "BudgetDialog/header" );

BudgetDialog::BudgetDialog(QWidget *parent) : QDialog(parent) {
  widget.setupUi( this );
  widget.entryarea->setEnabled( false );
  widget.deleter->setEnabled( false );

  model = new BudgetLimitRowModel( current );
  widget.limitsarea->setModel( model );

  AccountDelegate * ad = new AccountDelegate( false );
  widget.limitsarea->setItemDelegateForColumn( 0, ad );
  widget.limitsarea->setItemDelegateForColumn( 1, new MoneyDelegate() );
  widget.limitsarea->setItemDelegateForColumn( 2, new BoolDelegate() );

  connect( widget.newbudget, SIGNAL( clicked( ) ), SLOT( makeBudget( ) ) );
  connect( widget.budgetSelector,
      SIGNAL( itemActivated( QListWidgetItem * ) ),
      SLOT( loadBudget( QListWidgetItem * ) ) );
  QPushButton * btn = widget.editButtonBox->button( QDialogButtonBox::Save );
  btn->setEnabled( false );
  connect( btn, SIGNAL( clicked( ) ), SLOT( update( ) ) );
  connect( widget.deleter, SIGNAL( clicked( ) ), SLOT( remove( ) ) );

  widget.scaler->addItem( tr( "Month to Date" ), MonthToDate );
  widget.scaler->addItem( tr( "Running Month" ), RunningMonth );
  widget.scaler->addItem( tr( "Year to Date" ), YearToDate );
  widget.scaler->addItem( tr( "Running Year" ), RunningYear );

  refresh();
}

void BudgetDialog::makeBudget( ){
  Budget l;
  l.setName( tr( "New Budget" ) );
  BudgetMapper& mapper = QHacc::getInstance()->getBudgetMapper();
  mapper.save( l );

  QListWidgetItem * item = addItem( l );
  widget.budgetSelector->setCurrentItem( item );
}

void BudgetDialog::loadBudget( QListWidgetItem * idx ){
  if ( 0 != idx ) {
    BudgetMapper& mapper = QHacc::getInstance()->getBudgetMapper();
    int tid = idx->data( Qt::UserRole ).toInt( );

    current = mapper.getById( tid );
    model->setBudget( current );
    widget.namer->setText( current.getName() );
    int cidx = widget.scaler->findData( current.getTime() );
    widget.scaler->setCurrentIndex( cidx );
    widget.otherlimit->setText( current.getOtherLimit().toString() );
  }

  QPushButton * btn = widget.editButtonBox->button( QDialogButtonBox::Save );
  btn->setEnabled( 0 != idx );

  widget.deleter->setEnabled( 0 != idx );
  widget.entryarea->setEnabled( 0 != idx );
}

void BudgetDialog::update( ){
  current = model->getBudget();
  current.setName( widget.namer->text() );

  int idx = widget.scaler->currentIndex();
  QVariant data = widget.scaler->itemData( idx );
  current.setTime( static_cast<BudgetTime>( data.toInt() ) );

  QString str = widget.otherlimit->text();
  current.setOtherLimit( Money::getMoney( str ) );

  BudgetMapper& mapper = QHacc::getInstance()->getBudgetMapper();
  mapper.update( current );
  refresh();
  QSettings settings;
  settings.setValue( settingsKey,
                     widget.limitsarea->horizontalHeader()->saveState() );
}

void BudgetDialog::remove( ){
  BudgetMapper& mapper = QHacc::getInstance()->getBudgetMapper();
  mapper.remove( current );
  refresh( );
}

void BudgetDialog::refresh( ){
  widget.budgetSelector->clear( );

  QString empty;
  widget.namer->setText( empty );

  for( const Budget& t : QHacc::getInstance( )->getBudgetMapper().getAll() ) {
    addItem( t );
  }

  widget.deleter->setEnabled( false );
  widget.entryarea->setEnabled( false );
  QPushButton * btn = widget.editButtonBox->button( QDialogButtonBox::Save );
  btn->setEnabled( false );

  QSettings settings;
  widget.limitsarea->horizontalHeader()->
      restoreState( settings.value( settingsKey ).toByteArray() );
}

QListWidgetItem * BudgetDialog::addItem( const Budget& h ) {
  QListWidgetItem * item = new QListWidgetItem( widget.budgetSelector );
  item->setText( h.getName( ) );
  item->setData( Qt::UserRole, h.getId() );
  return item;
}
