#include "BudgetLimitRowModel.h"
#include "QHacc.h"
#include "AccountMapper.h"
#include "Logger.h"

BudgetLimitRowModel::BudgetLimitRowModel(const Budget& b, QObject *parent) :
  QAbstractTableModel( parent ), budget( b ){
}

int BudgetLimitRowModel::rowCount(const QModelIndex& idx) const{
  return ( idx.isValid() ? 0 : budget.getLimits().size() + 1 );
}

int BudgetLimitRowModel::columnCount(const QModelIndex&) const{
  return 3;
}

QVariant BudgetLimitRowModel::data(const QModelIndex& idx, int role ) const {
  if( idx.isValid() && ( role == Qt::DisplayRole || role == Qt::UserRole ) ){
    const int row = idx.row();
    const int col = idx.column();

    int count = 0;
    for( const auto& p : budget.getLimits() ){
      int aid = p.first;
      std::pair<Money, bool> pp = p.second;
      const Money& m = pp.first;
      bool inc = pp.second;

      if( row == count ){
        if( 0 == col ){
          if( Qt::DisplayRole == role ){
            Account a = QHacc::getInstance()->getAccountMapper().getById( p.first );
            return a.getName();
          }
          else{
             return aid;
          }
        }
        else if( 1 == col ){
            return ( Qt::DisplayRole == role ? QVariant( m.toString() ) :
                                               QVariant( m.getValue() ) );
        }
        else {
          return inc;
        }
      }

      count++;
    }
  }

  return QVariant();
}

void BudgetLimitRowModel::setBudget( const Budget& b ){
  budget = b;
  reset();
}

const Budget& BudgetLimitRowModel::getBudget() const{
  return budget;
}

QVariant BudgetLimitRowModel::headerData( int col, Qt::Orientation orient, int role ) const {
  if ( Qt::DisplayRole == role && Qt::Horizontal == orient ) {
    if( 0 == col ){
      return tr( "Account" );
    }
    else if( 1 == col ){
      return tr( "Limit", "budget limit" );
    }
    else {
      return tr( "+ Subs", "include sub accounts in budget" );
    }
  }

  return QVariant( );
}

bool BudgetLimitRowModel::setData( const QModelIndex & idx,
                                   const QVariant & value, int role ){
  bool ok = false;

  if ( idx.isValid( ) && Qt::EditRole == role ) {
    const int ROW = idx.row( );
    const int col = idx.column();
    int count = 0;
    for( const auto& p : budget.getLimits() ) {
      int aid = p.first;
      std::pair<Money, bool> pp = p.second;
      const Money& m = pp.first;
      bool inc = pp.second;

      if( ROW == count ){
        ok = true;

        if( 0 == col ){
          budget.addBudgetItem( value.toInt(), m, inc );
        }
        else if( 1 == col ){
          budget.addBudgetItem( aid, Money::getMoney( value.toString() ), inc );
        }
        else if ( 2 == col ){
          budget.addBudgetItem( aid, m, value.toBool() );
        }

        break;
      }

      count++;
    }

    if( !ok ){
      if( 0 == col ){
        if( value.toInt() > 0 ){
          budget.addBudgetItem( value.toInt(), Money() );
          ok = true;
          reset();
        }
      }
    }
  }

  return ok;
}

Qt::ItemFlags BudgetLimitRowModel::flags( const QModelIndex& idx ) const {
  if( 2 == idx.column() ){
    return ( Qt::ItemIsEditable | Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable );
  }
  else{
    return ( Qt::ItemIsEditable | Qt::ItemIsSelectable | Qt::ItemIsEnabled );
  }
}
