/*
 * File:   ReconcileSplitView.cpp
 * Author: ryan
 *
 * Created on December 6, 2011, 2:31 PM
 */

#include "ReconcileSplitView.h"


#include <QKeyEvent>
#include <QStringList>

#include "QHacc.h"
#include "AccountUtils.h"
#include "AccountMapper.h"
#include "Logger.h"
#include "FullTransaction.h"
#include "SplitMapper.h"
#include "DateDelegate.h"
#include "EngineNotifier.h"
#include "TransactionTypeFilter.h"

ReconcileSplitView::ReconcileSplitView( QWidget * p ) : TransactionViewer( p ),
 recofilter( Account() ){
  QStringList headers;
  headers << tr( "Date" ) << tr( "Memo" ) << tr( "Credit" ) << tr( "Debit" )
          << tr( "Rec", "reconciled field in account viewer" );
  setHeaderLabels( headers );

  connect( this, SIGNAL( itemClicked( QTreeWidgetItem *, int ) ),
           this, SLOT( toggle( QTreeWidgetItem *, int ) ) );

  setItemDelegateForColumn( 0, new DateDelegate( ) );

  EngineNotifier * notifier = QHacc::getInstance( )->getNotifier( );
  connect( notifier, SIGNAL( transactionAdded( const FullTransaction& ) ),
           SLOT( addTrans( const FullTransaction& ) ) );
  connect( notifier, SIGNAL( splitChanged( const Split&, const Split& ) ),
           SLOT( changeSplit( const Split&, const Split& ) ) );

  restoreSettings( "ReconcileWindow/state" );

  std::unique_ptr<TransactionFilter> uptr( new TransactionTypeFilter( Normal ) );
  filters.push_back( std::move( uptr ) );

  recofilter.setAntiReconciledState( Yes );
}

QList<Split> ReconcileSplitView::getSplits( ) {
  return splits.values( );
}

void ReconcileSplitView::keyPressEvent( QKeyEvent* key ) {
  if ( key->key( ) == Qt::Key_Space ) {
    key->accept( );
    toggle( currentItem( ), 0 );
  }
  else {
    TransactionViewer::keyPressEvent( key );
  }
}

void ReconcileSplitView::toggle( QTreeWidgetItem * item, int ) {
  int sid = item->data( 0, Qt::UserRole ).toInt( );
  SplitMapper& smap = QHacc::getInstance( )->getSplitMapper( );
  Split split = smap.getById( sid );
  if ( No == split.getReconciled( ) ) {
    split.setReconciled( Maybe, QDate::currentDate( ) );
    //item->setData( 4, Qt::DisplayRole, tr( "C", "cleared state" ) );
  }
  else {
    split.setReconciled( No, QDate::currentDate( ) );
    //item->setData( 4, Qt::DisplayRole, "" );
  }

  saveSettings( "ReconcileWindow/state" );
  smap.update( split );
  //splits[split.getId( )] = split;
  //emit reconcileChanged( );
}

void ReconcileSplitView::changeAccount(const Account &newone){
  recofilter.setAccount( newone );
  TransactionViewer::changeAccount( newone );
}

bool ReconcileSplitView::shouldIncludeTrans( const FullTransaction& ft ) {
  if( TransactionViewer::shouldIncludeTrans( ft ) ){
    return ( recofilter.accepts( ft ) && datefilter.accepts( ft ) );
  }

  return false;
}

void ReconcileSplitView::addTrans( const FullTransaction& ft ) {
  if ( shouldIncludeTrans( ft ) ) {
    QTreeWidgetItem * item = new QTreeWidgetItem( this );
    const Transaction& t = ft.getTransaction( );
    const Split& s = ft[account];
    splits[s.getId( )] = s;

    item->setData( 0, Qt::DisplayRole, t.getDate( ) );
    item->setData( 0, Qt::UserRole, s.getId( ) );
    item->setData( 0, DATEROLE, t.getDate( ) );

    items[s.getId( )] = item;
    changeSplit( s, s );
  }
}

void ReconcileSplitView::setEndDate( const QDate & date ) {
  ender = date;
  datefilter.setLatest( date );
  changeAccount( account );
}

void ReconcileSplitView::changeSplit( const Split&, const Split & newer ) {
  if ( splits.contains( newer.getId( ) ) ) {
    QTreeWidgetItem * item = items[newer.getId( )];
    item->setData( 1, Qt::DisplayRole, newer.getMemo( ) );
    item->setData( ( newer.getValue( ) < 0 ? 3 : 2 ),
                   Qt::DisplayRole, newer.getValue( ).toPositiveString( ) );
    splits[newer.getId( )] = newer;
    ReconcileState rs = newer.getReconciled( );
    if ( No == rs ) {
      item->setData( 4, Qt::DisplayRole, "" );
    }
    else if ( Maybe == rs ) {
      item->setData( 4, Qt::DisplayRole, tr( "C", "cleared state" ) );
    }

    if ( !loading ) {
      emit reconcileChanged( );
    }
  }
}

void ReconcileSplitView::clear(){
  TransactionViewer::clear();
  splits.clear();
  items.clear();
}
