/*
 * File:   ReconcileWindow.cpp
 * Author: ryan
 *
 * Created on December 4, 2011, 7:35 PM
 */

#include "ReconcileWindow.h"

#include <QKeyEvent>
#include <QPushButton>
#include <QSettings>

#include "QHacc.h"
#include "AccountUtils.h"
#include "AccountMapper.h"
#include "SplitMapper.h"
#include "Split.h"
#include "ReconcileSplitView.h"
#include "FullTransaction.h"
#include "Logger.h"
#include "EngineNotifier.h"

ReconcileWindow::ReconcileWindow( const Account& a, const Journal& j,
    QWidget * p ) : QDialog( p ), account( a ), journal( j ) {
  widget.setupUi( this );
  widget.dataentry->changeJournal( j );

  QDate closedate( QDate::currentDate( ) );
  AccountUtils& autils = QHacc::getInstance( )->getAccountUtils( );
  SplitMapper& smap = QHacc::getInstance( )->getSplitMapper( );

  Money recBal = autils.balanceOn( a, closedate, Reconciled );

  widget.dater->setDate( closedate );
  widget.opening->setText( recBal.toString( ) );

  QList<Split> splits;

  for( const Split& s : smap.getAll( a, j, closedate ) ) {
    if ( Yes != s.getReconciled( ) ) {
      splits << s;
    }
  }

  widget.splitview->setEndDate( widget.dater->date( ) );
  widget.splitview->setJournal( journal );
  widget.splitview->changeAccount( account );
  widget.dataentry->setVisible( false );

  connect( widget.balancer, SIGNAL( textChanged( const QString& ) ),
      SLOT( recalcBalance( ) ) );
  connect( widget.splitview, SIGNAL( reconcileChanged( ) ),
      SLOT( recalcBalance( ) ) );

  connect( widget.dater, SIGNAL( dateChanged( const QDate& ) ),
      widget.splitview, SLOT( setEndDate( const QDate& ) ) );

  QPushButton * btn = widget.buttonBox->button( QDialogButtonBox::Ok );
  connect( btn, SIGNAL( clicked( ) ), SLOT( upgradeReconciled( ) ) );
  connect( btn, SIGNAL( clicked( ) ), SLOT( saveSettings( ) ) );

  connect( widget.dater, SIGNAL( dateChanged( const QDate& ) ),
      SLOT( recalcBalance( ) ) );

  recalcBalance( );
  restoreSettings( );
}

void ReconcileWindow::recalcBalance( ) {
  bool rightplus = account.isRightPlus( );

  Money opening = Money::getMoney( widget.opening->text( ) );
  Money stmt = Money::getMoney( widget.balancer->text( ) );
  Money diff( stmt - opening );
  Money running;

  for( const Split& s : widget.splitview->getSplits( ) ) {
    if ( Maybe == s.getReconciled( ) ) {
      running += s.getValue( );
    }
  }

  if ( rightplus ) {
    running.negate( );
  }

  diff -= running;

  widget.difference->setText( diff.toString( ) );
}

void ReconcileWindow::upgradeReconciled( ) {
  SplitMapper& smap = QHacc::getInstance( )->getSplitMapper( );
  QList<Split> splits = widget.splitview->getSplits();
  widget.splitview->clear();

  for( Split& s : splits ) {
    if ( Maybe == s.getReconciled( ) ) {
      s.setReconciled( Yes );
    }
    smap.update( s );
  }

  AccountMapper& amap = QHacc::getInstance()->getAccountMapper();
  amap.updateBalances( account );
}

void ReconcileWindow::saveSettings( ) {
  QSettings settings;
  settings.setValue( "Reconciler/Window", saveGeometry( ) );
  widget.splitview->saveSettings( "Reconciler/View" );
}

void ReconcileWindow::restoreSettings( ) {
  QSettings settings;
  restoreGeometry( settings.value( "Reconciler/Window" ).toByteArray( ) );
  widget.splitview->restoreSettings( "Reconciler/View" );

  widget.dater->setDisplayFormat( settings.value( DateFormat ).toString( ) );
}

void ReconcileWindow::newTransaction(){
  Transaction t;
  t.setDate( widget.dater->date() );
  t.setJournalId( journal.getId() );

  Split s;
  s.setReconciled( Maybe );
  s.setAccountId( account.getId() );

  QList<Split> tsplits;
  tsplits << s;

  widget.dataentry->edit( FullTransaction( t, tsplits ) );
  widget.dataentry->setVisible( true );
}
