/*
 * File:   SplitDialog.cpp
 * Author: ryan
 *
 */

#include "SplitDialog.h"
#include "SplitModel.h"

#include <QFileDialog>
#include <QDirModel>
#include <QCompleter>
#include <QPushButton>

#include <iostream>

SplitDialog::SplitDialog( QWidget * p ) : QDialog( p ) {
  widget.setupUi( this );
  setAttribute( Qt::WA_DeleteOnClose, false );
  forceBalance = true;
  connect( widget.tree->model(), SIGNAL( differenceChanged( const Money&, const Money& ) ),
           SLOT( checkOk() ) );
}

void SplitDialog::setSplits( const QList<Split>& s ) {
  widget.tree->setSplits( s );
}

const QList<Split>& SplitDialog::splits() const {
  return widget.tree->getSplits();
}

void SplitDialog::setDefaultMemo( const QString& s ){
  widget.tree->changeMemo( s );
}

void SplitDialog::mustBeBalanced( bool b ){
  forceBalance = b;
}

void SplitDialog::checkOk(){
  QString msg = "Ok";
  QPushButton * btn = widget.buttonBox->button( QDialogButtonBox::Ok );

  if( forceBalance ){
    Money m = widget.tree->getDifference();
    btn->setEnabled( m.isZero() );

    if( !m.isZero() ){
      msg = QString( "Unbalanced: %1" ).arg( m.toString() );
    }
  }

  btn->setText( msg );
}
