/*
 * File:   SplitModel.h
 * Author: ryan
 *
 * Created on November 27, 2011, 9:07 PM
 */

#ifndef SPLITMODEL_H
#define	SPLITMODEL_H

#include <QAbstractTableModel>
#include <QStringList>

#include "Split.h"

class SplitModel : public QAbstractTableModel {
  Q_OBJECT
public:
  SplitModel( QObject * p = 0 );
  virtual ~SplitModel( ){}

  /**
   * Sets the splits and the default account for new splits
   * @param splits the existing splits
   * @param acctid the account to put new splits into
   */
  virtual void setSplits( QList<Split> splits );
  const QList<Split>& getSplits( ) const;

  int rowCount( const QModelIndex &parent = QModelIndex( ) ) const;
  int columnCount( const QModelIndex &parent = QModelIndex( ) ) const;

  virtual QVariant data( const QModelIndex &index,
      int role = Qt::DisplayRole ) const;
  virtual QVariant headerData( int section, Qt::Orientation orientation,
      int role = Qt::DisplayRole ) const;
  virtual Qt::ItemFlags flags( const QModelIndex & index ) const;
  virtual bool setData( const QModelIndex & index, const QVariant & value,
      int role = Qt::EditRole );
  Money getDifference( ) const;

signals:
  void differenceChanged( const Money& older, const Money& newer );

public slots:
  /**
   * Adds a balancing split (but with no account).
   */
  void addSplit( );
  void changeMemo( const QString& str );

protected:
  QStringList headers;
  QList<Split> splits;
  void figureDifference( );
  QString defaultMemo;
  Money difference;
};

#endif	/* SPLITMODEL_H */
