/*
 * File:   SplitsView.cpp
 * Author: ryan
 *
 * Created on November 30, 2011, 2:39 PM
 */

#include "SplitsView.h"
#include "SplitModel.h"
#include "AccountDelegate.h"
#include "ReconcileDelegate.h"
#include "MoneyDelegate.h"
#include "Logger.h"

#include <QSettings>
#include <QHeaderView>

SplitsView::SplitsView( QWidget * p ) : QTreeView( p ),
  mmodel( new SplitModel() ), adele( new AccountDelegate() ),
  rdele( new ReconcileDelegate() ), mdele( new MoneyDelegate() ) {

  setModel( mmodel.get() );
  setItemDelegateForColumn( 4, rdele.get() );
  setItemDelegateForColumn( 0, adele.get() );
  setItemDelegateForColumn( 3, mdele.get() );
  setItemDelegateForColumn( 2, mdele.get() );
}

SplitsView::~SplitsView( ) {
}

void SplitsView::addSplit( ) {
  mmodel->addSplit( );
}

void SplitsView::changeMemo( const QString& str ) {
  mmodel->changeMemo( str );
}

void SplitsView::setSplits( QList<Split> splits ) {
  mmodel->setSplits( splits );
}

Money SplitsView::getDifference( ) const {
  return mmodel->getDifference( );
}

const QList<Split>& SplitsView::getSplits( ) const {
  return mmodel->getSplits( );
}

//void SplitsView::edit ( const QModelIndex & i ){
//  setCurrentIndex( i );
//  QTreeView::edit( i );
//}

//bool SplitsView::edit ( const QModelIndex & i, EditTrigger t, QEvent * e ){
//  setCurrentIndex( i );
//  return QTreeView::edit( i, t, e );
//}

void SplitsView::saveSettings( const QString& key ) const {
  QSettings settings;
  settings.setValue( key, header()->saveState());
}

void SplitsView::restoreSettings( const QString& key ) {
  QSettings settings;
  header()->restoreState(settings.value( key ).toByteArray() );
}

QModelIndex SplitsView::moveCursor( CursorAction action,
                                    Qt::KeyboardModifiers mods ){
  QModelIndex index = currentIndex();
  if( QAbstractItemView::MoveNext == action ) {
    if( 4 == index.column() ){
      return mmodel->index( index.row() + 1, 0, index.parent() );
    }
    else{
      return mmodel->index( index.row(), index.column() + 1, index.parent() );
    }
  }
  else if( QAbstractItemView::MovePrevious == action ) {
    if( 0 == index.column() ){
      return mmodel->index( index.row() -1, 0, index.parent() );
    }
    else{
      return mmodel->index( index.row(), index.column() - 1, index.parent() );
    }
  }

  return QTreeView::moveCursor( action, mods );
}
