/*
 * File:   TransactionViewer.cpp
 * Author: ryan
 *
 * Created on December 13, 2011, 6:23 PM
 */

#include "TransactionViewer.h"

#include <QSettings>
#include <QHeaderView>

#include "QHacc.h"
#include "FullTransaction.h"
#include "TransactionFilter.h"

const int TransactionViewer::DATEROLE = Qt::UserRole + 1;

TransactionViewer::TransactionViewer( QWidget * p ) : QTreeWidget( p ) {
}

void TransactionViewer::setJournal( const Journal& j ) {
  journal = j;
}

void TransactionViewer::setAccount( const Account& a ) {
  changeAccount( a );
}

const Journal& TransactionViewer::getJournal( ) const {
  return journal;
}

void TransactionViewer::addCustomFilter( std::unique_ptr<TransactionFilter> tf ){
  customfilters.push_back( std::move( tf ) );
  changeAccount( account );
}

void TransactionViewer::clearCustomFilters(){
  customfilters.clear();
  changeAccount( account );
}

void TransactionViewer::saveSettings( const QString& key ) {
  QSettings settings;
  settings.setValue( key, header( )->saveState( ) );
}

void TransactionViewer::restoreSettings( const QString& key ) {
  QSettings settings;
  header( )->restoreState( settings.value( key ).toByteArray( ) );
}

bool TransactionViewer::shouldIncludeTrans( const FullTransaction& ft ){
  bool ok = true;
  for( const auto& filter : filters ){
    ok = ok && filter->accepts( ft );
  }

  for( const auto& filter : customfilters ){
    ok = ok && filter->accepts( ft );
  }

  return ok;
}

void TransactionViewer::changeAccount( const Account& newone ) {
  loading = true;
  clear( );
  cleanup();
  setSortingEnabled( false );
  account = newone;
  QHacc * qhacc = QHacc::getInstance( );
  for( const FullTransaction& ft : qhacc->getTransactions( newone, journal ) ) {
    addTrans( ft );
  }

  expandAll( );
  setSortingEnabled( true );
  loading = false;
}

void TransactionViewer::cleanup(){}
