#include "DateRangeFilter.h"

#include "FullTransaction.h"
#include "Transaction.h"

DateRangeFilter::DateRangeFilter( const QDate& earliest, const QDate& latest ){
  setRange( earliest, latest );
}

DateRangeFilter::DateRangeFilter( const DateRangeFilter& orig ) :
  TransactionFilter( orig ), earliestDate( orig.earliestDate),
  latestDate( orig.latestDate ){}

DateRangeFilter& DateRangeFilter::operator=( const DateRangeFilter& orig ){
  if( this != &orig ){
    earliestDate = orig.earliestDate;
    latestDate = orig.latestDate;
  }

  return *this;
}

void DateRangeFilter::setRange( const QDate& sd, const QDate& ed ){
  setEarliest( sd );
  setLatest( ed );
}

void DateRangeFilter::setEarliest( const QDate& sd ){
  earliestDate = sd;
}

void DateRangeFilter::setLatest( const QDate& sd ){
  latestDate = sd;
}

bool DateRangeFilter::iaccept( const FullTransaction& ft ){
  QDate date( ft.getTransaction().getDate() );

  bool ok = true;

  if( earliestDate.isValid() ){
    ok = ( ok && date >= earliestDate );
  }

  if( latestDate.isValid() ){
    ok = ( ok && date <= latestDate );
  }

  return ok;
}
