/*
 * File:   BalanceBase.cpp
 * Author: ryan
 *
 * Created on November 21, 2011, 3:13 PM
 */

#include <QHash>

#include "Balance.h"

#include "qwt_plot.h"
#include "qwt_plot_curve.h"
#include "QHacc.h"
#include "TransactionUtils.h"
#include "AccountUtils.h"
#include "Logger.h"

BalanceBase::BalanceBase( const Account& a, const QDate& b, const QDate& e )
: AccountGenerator( a, b, e ) {}

BalanceBase::BalanceBase( const BalanceBase& pg ) : AccountGenerator( pg ) {}

BalanceBase& BalanceBase::operator=(const BalanceBase& pg ) {
  if ( this != &pg ) {
    AccountGenerator::operator=( pg );
  }
  return *this;
}

std::unique_ptr<QwtPlotItem> BalanceBase::getPlot( const QPen& pen ) {
  setPen( pen );
  QHacc * engine = QHacc::getInstance( );
  const AccountUtils& autils = engine->getAccountUtils( );
  QList<FullTransaction> trans =
      engine->getTransactionUtils( ).getTransactions( account, Journal( ),
      stop, start );

  std::unique_ptr<QwtPlotCurve> curve(
      new QwtPlotCurve( QObject::tr( "%1 Balance", "Plot legend text" ).
      arg( account.getName( ) ) ) );
  const QString DATEFORMAT = dateFormat();
  QHash<QString, Money> monies;
  QDate tick = start;
  while ( tick <= stop ) {
    monies.insert( tick.toString( DATEFORMAT ), Money( ) );
    tick = incrementTick( tick );
  }

  const int ACCOUNTID = account.getId( );

  for( const FullTransaction& t : trans ) {
    QString df = t.getTransaction( ).getDate( ).toString( DATEFORMAT );

    for( const Split& s : t.getSplits( ) ) {
      if ( ACCOUNTID == s.getAccountId( ) ) {
        Money sval( s.getValue() );
        if( account.isRightPlus()){
          sval.negate();
        }

        monies[df] += sval;
      }
    }
  }

  double * yvals = new double[monies.size( )];
  double * xvals = new double[monies.size( )];
  Money runningBalance;
  if( sumThroughTicks() ){
    runningBalance = autils.balanceOn( account, start );
  }
  tick = start;

  int idx = 0;
  while ( tick <= stop ) {
    if( sumThroughTicks() ){
      runningBalance += monies[tick.toString( DATEFORMAT )];
    }
    else{
      runningBalance = monies[tick.toString( DATEFORMAT )];
    }

    Logger::debug( QString( "for %1, adding %2" ).arg( tick.toString( ) ).
        arg( monies[tick.toString( DATEFORMAT )].toString( ) ) );

    xvals[idx] = idx;
    Money acctbal( runningBalance );

    yvals[idx++] = acctbal.toDouble( );
    tick = incrementTick( tick );
  }

  for ( int i = 0; i < monies.size( ); i++ ) {
    Logger::debug( QString( "(month,balance): (%1,%2)" ).
        arg( xvals[i] ).arg( yvals[i] ) );
  }

  curve->setSamples( xvals, yvals, monies.size( ) );
  curve->setPen( pen );
  delete []xvals;
  delete [] yvals;
  return std::move( curve );
}
/**
std::unique_ptr<QwtPlotItem> BalanceBase::getPlot( const QPen& pen ) {
  setPen( pen );
  QHacc * engine = QHacc::getInstance( );
  const AccountUtils& autils = engine->getAccountUtils( );
  QList<FullTransaction> trans =
      engine->getTransactionUtils( ).getTransactions( account, Journal( ),
      stop, start );

  std::unique_ptr<QwtPlotHistogram> curve(
      new QwtPlotHistogram( QObject::tr( "%1 Balance", "Plot legend text" ).
      arg( account.getName() ) ) );

  QVector<QwtIntervalSample> vect;
  const QString DATEFORMAT = dateFormat();
  QHash<QString, Money> monies;
  QDate tick = start;
  while ( tick <= stop ) {
    monies.insert( tick.toString( DATEFORMAT ), Money( ) );
    tick = incrementTick( tick );
  }

  const int ACCOUNTID = account.getId( );

  for( const FullTransaction& t : trans ) {
    QString df = t.getTransaction( ).getDate( ).toString( DATEFORMAT );

    for( const Split& s : t.getSplits( ) ) {
      if ( ACCOUNTID == s.getAccountId( ) ) {
        Money sval( s.getValue() );
        if( account.isRightPlus()){
          sval.negate();
        }

        monies[df] += sval;
      }
    }
  }

  Money runningBalance;
  if( sumThroughTicks() ){
    runningBalance = autils.balanceOn( account, start );
  }
  tick = start;

  int idx = 0;
  while ( tick <= stop ) {
    if( sumThroughTicks() ){
      runningBalance += monies[tick.toString( DATEFORMAT )];
    }
    else{
      runningBalance = monies[tick.toString( DATEFORMAT )];
    }

    float istart = idx + 0.07;
    float iend = idx + 0.93;
    vect << QwtIntervalSample( runningBalance / 100, istart, iend );
    tick = incrementTick( tick );
    idx++;
  }

  QwtIntervalSeriesData * data = new QwtIntervalSeriesData( vect );
  curve->setBrush( pen.color() );
  curve->setData( data );
  curve->setPen( pen );
  return std::move( curve );
}
*/
