#include "Budgeting.h"

#include "Budget.h"
#include "Logger.h"
#include "QHacc.h"
#include "AccountMapper.h"
#include "AccountUtils.h"
#include "BudgetUtils.h"

#include <utility>

#include <QVector>

#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_histogram.h>
#include <qwt_series_data.h>

Budgeting::Budgeting( const Budget& b, bool actuals, const QDate &endTime )
: PlotGenerator( QDate(), endTime ), budget( b ), actual( actuals ) {
}

Budgeting::Budgeting( bool actuals, const QDate &endTime )
: PlotGenerator( QDate(), endTime ), actual( actuals ) {
}

QString Budgeting::getTitle() const {
  return budget.getName();
}

std::unique_ptr<QwtPlotItem> Budgeting::getPlot( const QPen& pen ){
  setPen( pen );
  std::unique_ptr<QwtPlotHistogram> curve(
      new QwtPlotHistogram( QObject::tr( "%1 Budget", "Plot legend text" ).
      arg( budget.getName() ) ) );

  AccountMapper& amap = QHacc::getInstance()->getAccountMapper();
  BudgetUtils& butils = QHacc::getInstance()->getBudgetUtils();

  std::map<Account, Money> bmonies =  butils.budgetBreakdown( budget, stop );
  QVector<QwtIntervalSample> vect;

  Money budgetamt;
  Money runningamt;

  int idx = 0;
  for( const auto& p : budget.getLimits() ){
    int aid = p.first;
    std::pair<Money, bool> pp = p.second;
    Money blimit = pp.first;
    budgetamt += blimit;

    if( actual ){
      Account acct = amap.getById( aid );
      Money money = bmonies.at( acct );

      float istart = idx + 0.05;
      float iend = idx + 0.48;
      runningamt += money;
      vect << QwtIntervalSample( money.getValue() / 100, istart, iend );
    }
    else {
      float istart = idx + 0.52;
      float iend = idx + 0.95;
      runningamt += blimit;
      vect << QwtIntervalSample( blimit / 100, istart, iend );
    }

    idx++;
  }

  if( budget.getOtherLimit().isNonZero() ){
    Money money = bmonies.at( Account() );

    if( actual ){
      float istart = idx + 0.05;
      float iend = idx + 0.48;
      runningamt += money;

      vect << QwtIntervalSample( money.getValue() / 100, istart, iend );
    }
    else {
      float istart = idx + 0.52;
      float iend = idx + 0.95;
      Money blimit = budget.getOtherLimit();

      runningamt += blimit;
      budgetamt += blimit;
      vect << QwtIntervalSample( blimit / 100, istart, iend );
    }
  }


  QwtIntervalSeriesData * data = new QwtIntervalSeriesData( vect );
  curve->setData( data );
  curve->setPen( pen );
  curve->setBrush( QBrush( pen.color() ) );

  double ramt = runningamt.getValue();
  double bamt = budgetamt.getValue();
  pct = QString( "%1%" ).arg( 100 * ramt / bamt, 0, 'f', 1 );

  return std::move( curve );
}

std::map<QString, std::pair<QColor, QString> > Budgeting::legendMap() const {
  std::map<QString, std::pair<QColor, QString> > ret;

  if( actual ){
    ret.insert( std::make_pair( QObject::tr( "Actual", "actual expenses vs budget" ),
                                std::make_pair( color, pct ) ) );
  }
  else {
    ret.insert( std::make_pair( QObject::tr( "Budget", "budgeted expenses" ),
                                std::make_pair( color, pct ) ) );
  }

  return ret;
}

void Budgeting::setBudget( const Budget& b ){
  budget = b;
}
