#include "BudgetingWindow.h"

#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_legend.h>

#include "Budgeting.h"
#include "AccountLabeler.h"
#include "QHacc.h"
#include "BudgetMapper.h"
#include "BudgetUtils.h"
#include "AccountMapper.h"
#include "AccountUtils.h"
#include "Logger.h"

BudgetingWindow::BudgetingWindow( QWidget * p )
: PlotWindow( new Budgeting(), p ){
  init();
}

void BudgetingWindow::init(){
  widget.startlabel->setVisible( false );
  widget.startdate->setVisible( false );
  widget.plotButton->setVisible( false );
  QDate endOfMonth = QDate::currentDate();

  widget.enddate->setDate( endOfMonth.addDays( endOfMonth.daysInMonth() -
                                               endOfMonth.day() ) );

  connect( widget.enddate, SIGNAL( dateChanged( const QDate& ) ),
           SLOT( replot() ) );

  widget.addA->setText( tr( "View" ) );

  widget.accounts->clear();
  BudgetMapper& bmap = QHacc::getInstance()->getBudgetMapper();
  for( const auto& p : bmap.getAll() ){
    widget.accounts->addItem( p.getName(), p.getId() );
  }

  actual = new Budgeting( true );
  budgeted = new Budgeting( false );
}

void BudgetingWindow::add(){
  int idx = widget.accounts->currentIndex();
  int bid = widget.accounts->itemData( idx ).toInt();
  BudgetMapper& bmap = QHacc::getInstance()->getBudgetMapper();
  Budget budget = bmap.getById( bid );

  actual->setBudget( budget );
  budgeted->setBudget( budget );
  replot();
}

TimeScale BudgetingWindow::timescale() const {
  return ACCOUNTS;
}


void BudgetingWindow::replot( ) {
  const int NUM( budgeted->getBudget().getLimits().size() );
  if( 0 == NUM ){
    return;
  }

  QDate starter = widget.startdate->date( );
  QDate ender = widget.enddate->date( );

  AccountLabeler * drawer = new AccountLabeler();

  widget.qwtPlot->setAxisScaleDraw( QwtPlot::xBottom, drawer );
  widget.qwtPlot->setAxisLabelRotation( QwtPlot::xBottom, 50.0 );

  widget.qwtPlot->detachItems();
  widget.qwtPlot->setAxisLabelAlignment( QwtPlot::xBottom,
      Qt::AlignRight | Qt::AlignBottom );

  AccountMapper& amap = QHacc::getInstance()->getAccountMapper();
  clearLegend();
  Budgeting * pgs[] = { budgeted, actual };
  QColor colors[] = { Qt::darkCyan, Qt::darkMagenta };
  for( int i = 0; i < 2; i++ ){
    QPen pen;
    pen.setColor( colors[i] );
    pen.setWidth( 3 );
    Budgeting * pg = pgs[i];

    const Budget& b = pg->getBudget();
    drawer->clear();
    for( const auto& p : b.getLimits() ){
      Account a = amap.getById( p.first );
      QString lbl = a.getName();
      if( p.second.second ){
        lbl.append( "+" );
      }

      drawer->addLabel( lbl );
    }

    if( b.getOtherLimit().isNonZero() ){
      drawer->addLabel( tr( "Other Accounts" ) );
    }

    pg->setWindow( starter, ender );
    std::unique_ptr<QwtPlotItem> item = pg->getPlot( pen );
    item.release()->attach( widget.qwtPlot );

    for( const auto& lm : pg->legendMap() ){
      const std::pair<QColor, QString>& dato = lm.second;
      QLabel * lbl = new QLabel( dato.second );
      lbl->setAlignment( Qt::AlignCenter );
      QColor foreg = Qt::white;

      QString style = QString("QLabel{background-color: %1; color: %2; padding: 2px}")
          .arg( dato.first.name() ).arg( foreg.name() );
      lbl->setStyleSheet( style );
      lbl->setFixedWidth( 50 );
      widget.legendlayout->addRow( lm.first, lbl );
    }
  }

  QwtScaleDiv * div = widget.qwtPlot->axisScaleDiv( QwtPlot::xBottom );
  div->setInterval( 0, drawer->count() );
  widget.qwtPlot->setAxisMaxMajor( QwtPlot::xBottom, drawer->count() );

  QwtPlotGrid * grid = new QwtPlotGrid( );
  grid->enableY( true );
  grid->enableX( false );
  QPen pen( Qt::DotLine );
  pen.setColor( Qt::gray );
  grid->setPen( pen );
  grid->attach( widget.qwtPlot );
  widget.qwtPlot->setAxisMaxMinor( QwtPlot::xBottom, 0 );
  widget.qwtPlot->replot( );
}
