/*
 * File:   PlotGenerator.cpp
 * Author: ryan
 *
 * Created on November 21, 2011, 4:15 PM
 */

#include "PlotGenerator.h"

PlotGenerator::PlotGenerator( const QDate& b, const QDate& e )
: start( b ), stop( e ){
  if ( start.isNull( ) ) {
    start = stop.addYears( -1 );
  }
}

PlotGenerator::PlotGenerator( const PlotGenerator& pg ) : start( pg.start ),
  stop( pg.stop ) {
}

PlotGenerator& PlotGenerator::operator=( const PlotGenerator& pg ) {
  if ( this != &pg ) {
    start = pg.start;
    stop = pg.stop;
  }
  return *this;
}

void PlotGenerator::setWindow( const QDate& begin, const QDate& end ) {
  start = begin;
  stop = end;
}

void PlotGenerator::setPen( const QPen& p ){
  color = p.color();
}




AccountGenerator::AccountGenerator( const Account& a, const QDate& begin,
  const QDate& end ) : PlotGenerator( begin, end ), account( a ){
}

AccountGenerator::AccountGenerator( const AccountGenerator& pg )
: PlotGenerator( pg ), account( pg.account ){}

AccountGenerator& AccountGenerator::operator=( const AccountGenerator& pg ){
  if( this != &pg ){
    PlotGenerator::operator=( pg );
    account = pg.account;
  }

  return *this;
}

QString AccountGenerator::getTitle( ) const{
  return account.getName();
}

std::map<QString, std::pair<QColor, QString> > AccountGenerator::legendMap() const {
  std::map<QString, std::pair<QColor, QString> > ret;
  ret.insert( std::make_pair( account.getName(), std::make_pair( color, "" ) ) );
  return ret;
}
