/*
 * File:   PlotGenerator.h
 * Author: ryan
 *
 * Created on November 21, 2011, 4:06 PM
 */

#ifndef PLOTGENERATOR_H
#define	PLOTGENERATOR_H

#include <QDate>
#include <QPen>

#include <memory>
#include <utility>
#include <map>

#include "Account.h"

class QwtPlotItem;

/**
 * A data generator for a single account over a specified window
 * @param a
 * @param start
 * @param end
 */
class PlotGenerator {
public:
  PlotGenerator( const QDate& begin = QDate( ),
      const QDate& end = QDate::currentDate( ) );
  PlotGenerator( const PlotGenerator& pg );
  PlotGenerator& operator=( const PlotGenerator& pg );
  virtual ~PlotGenerator( ){}

  virtual void setWindow( const QDate& start, const QDate& end );
  virtual QString getTitle( ) const = 0;

  /**
   * Sets the pen to use when getting the plot. Also, may define the color
   * to return in the legendMap
   */
  virtual void setPen( const QPen& pen = QPen() );

  /**
   * Creates a new QwtPlotCurve with the given pen. This function should also
   * call {@link #setPen}.
   * @return a curve of the data
   */
  virtual std::unique_ptr<QwtPlotItem> getPlot( const QPen& pen = QPen( ) ) = 0;
  virtual std::map<QString, std::pair<QColor, QString> > legendMap() const = 0;

protected:
  QDate start;
  QDate stop;
  QColor color;
};

class AccountGenerator : public PlotGenerator {
  public:
  AccountGenerator( const Account& a, const QDate& begin = QDate( ),
      const QDate& end = QDate::currentDate( ) );
  AccountGenerator( const AccountGenerator& pg );
  AccountGenerator& operator=( const AccountGenerator& pg );
  virtual ~AccountGenerator( ){}
  virtual QString getTitle( ) const;
  virtual std::map<QString, std::pair<QColor, QString> > legendMap() const;

protected:
  Account account;
};

#endif	/* PLOTGENERATOR_H */

