/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2010 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************** * * * * * * * * * * * * **************************/

#include <QApplication>
#include <QSettings>

#include <QSqlDatabase>
#include <QString>
#include <QSqlError>
#include <QSqlQuery>
#include <QFile>
#include <QTimer>
#include <QTranslator>
#include <QLibraryInfo>

#include <cstdlib>
#include <iostream>

#include "Logger.h"
#include "QHacc.h"

#include "MainWindow.h"
#include "StartupDialog.h"

void initSettings( QSettings& settings ) {
  if ( !settings.contains( CurrencySeparator ) ) {
    settings.setValue( CurrencySeparator, "." );
  }
  if ( !settings.contains( AccountSeparator ) ) {
    settings.setValue( AccountSeparator, "::" );
  }
  if ( !settings.contains( OpenJournalId ) ) {
    settings.setValue( OpenJournalId, "1" );
  }
  if ( !settings.contains( DateFormat ) ) {
    settings.setValue( DateFormat, "MM/dd/yyyy" );
  }
  if ( !settings.contains( CurrencySymbol ) ) {
    settings.setValue( CurrencySymbol, "$" );
  }
}

void installTranslation( QApplication * app ){
  QTranslator qtTranslator;
  qtTranslator.load( "qt_" + QLocale::system().name(),
                     QLibraryInfo::location( QLibraryInfo::TranslationsPath ) );
  app->installTranslator(&qtTranslator);

  char * envdb = getenv( "LANG_DIR" );
  QString qmloc;
  if( NULL != envdb ){
    // if passed as an environment variable, use it
    qmloc = QString( envdb );
    if( !qmloc.endsWith("/") ){
      qmloc += "/";
    }
  }
  else{
    qmloc = QString( "%1/../qm/").arg( QApplication::applicationDirPath() );
  }

  QTranslator myappTranslator;
  QString qmfile = QString( "qhacc.%1" ).arg( QLocale::system().name() );
  Logger::debug( QString( "using translation: %1%2" ).arg( qmloc ).arg( qmfile) );
  if( myappTranslator.load( qmfile, qmloc ) ){
    app->installTranslator( &myappTranslator );
  }
  else{
    Logger::debug( "translation not found" );
  }
}

int main( int argc, char *argv[] ) {
  // initialize resources, if needed
  // Q_INIT_RESOURCE(resfile);
  QApplication app( argc, argv );
  if ( app.styleSheet( ).isEmpty( ) ) {
    QFile qss( ":/stylesheet.qss" );
    qss.open( QIODevice::ReadOnly );
    app.setStyleSheet( qss.readAll( ) );
    qss.close( );
  }

  QCoreApplication::setOrganizationDomain( "ostrich-emulators.com" );
  QCoreApplication::setOrganizationName( "Ostrich Emulators" );
  QCoreApplication::setApplicationName( "QHacc4" );
  QSettings settings;
  initSettings( settings );

  Logger::setLevel( Debug );

    QTranslator qtTranslator;
  qtTranslator.load( "qt_" + QLocale::system().name(),
                     QLibraryInfo::location( QLibraryInfo::TranslationsPath ) );
  app.installTranslator(&qtTranslator);

  char * qenvdb = getenv( "LANG_DIR" );
  QString qmloc;
  if( NULL != qenvdb ){
    // if passed as an environment variable, use it
    qmloc = QString( qenvdb );
    if( !qmloc.endsWith("/") ){
      qmloc += "/";
    }
  }
  else{
    qmloc = QString( "%1/../qm/").arg( QApplication::applicationDirPath() );
  }

  QTranslator myappTranslator;
  QString qmfile = QString( "qhacc.%1" ).arg( QLocale::system().name() );
  Logger::debug( QString( "using translation: %1%2" ).arg( qmloc ).arg( qmfile) );
  if( myappTranslator.load( qmfile, qmloc ) ){
    app.installTranslator( &myappTranslator );
  }
  else{
    Logger::debug( "translation not found" );
  }


  QStringList args = app.arguments( );
  QString dbloc = settings.value( Database ).toString( ); // empty for new setup
  char * envdb = getenv( "QHACC_DB" );
  if( NULL != envdb ){
      // if passed as an environment variable, use it
      dbloc = QString( envdb );
  }
  if ( args.count( ) > 1 ) {
    // the command line db location trumps all else
    dbloc = args.at( 1 );
  }

  // db location not passed in on command line and not in the config file,
  // so get it from the user
  if ( dbloc.isEmpty( ) ) {
    StartupDialog dlg;
    int response = dlg.exec( );
    if ( QDialog::Accepted == response ) {
      dbloc = dlg.getDb( );
      settings.setValue( Database, dbloc );
    }
    else {
      Logger::fatal( "Setup canceled!" );
      exit( 1 );
    }
  }

  Logger::debug( QString( "Database: %1" ).arg( dbloc ) );
  QSqlDatabase sqldb = QSqlDatabase::addDatabase( "QSQLITE" );
  sqldb.setDatabaseName( dbloc );
  if ( !sqldb.open( ) ) {
    Logger::fatal( QString( "Could not connect to database: %1!" ).
        arg( dbloc ) );
    exit( 1 );
  }

  QHacc::init( sqldb );
  MainWindow * mw = new MainWindow( );
  mw->show( );
  return app.exec( );
}
